import 'package:loyalty_core_flutter/model/loyalty_configuration.dart';
import 'package:loyalty_core_flutter/model/member_info.dart';
import 'package:loyalty_core_flutter/model/membership_qr.dart';
import 'package:loyalty_core_flutter/model/netword_config_result.dart';
import 'package:loyalty_core_flutter/model/register_member_request.dart';
import 'package:loyalty_core_flutter/model/status.dart';
import 'package:loyalty_core_flutter/model/transaction_history.dart';
import 'package:loyalty_core_flutter/model/transaction_history_request.dart';

import '../model/transaction_type.dart';
import 'loyalty_api.dart';

class ModelConverter {
  ModelConverter._();

  static LoyaltyConfiguration toModelLoyaltyConfiguration(
      FlutterLoyaltyConfiguration flutterLoyaltyConfiguration) {
    return LoyaltyConfiguration(
        flutterLoyaltyConfiguration.url ?? "",
        flutterLoyaltyConfiguration.bffUrl ?? "",
        flutterLoyaltyConfiguration.merchantCode ?? "",
        flutterLoyaltyConfiguration.clientId ?? "",
        flutterLoyaltyConfiguration.iamAudience ?? "",
        flutterLoyaltyConfiguration.loggingEnabled ?? false,
        flutterLoyaltyConfiguration.linkMembershipUrl ?? "",
        flutterLoyaltyConfiguration.registerMemberUrl ?? "",
        flutterLoyaltyConfiguration.allowLinkMembership ?? false,
        flutterLoyaltyConfiguration.allowRegisterMembership ?? false,
        flutterLoyaltyConfiguration.membershipCodeFormat ?? "");
  }

  static MemberInfo toModelMemberInfo(FlutterMemberInfo flutterMemberInfo) {
    CardType? defaultCardType;
    if (flutterMemberInfo.defaultCardType != null) {
      if (flutterMemberInfo.defaultCardType == FlutterCardType.memberCard) {
        defaultCardType = CardType.memberCard;
      } else {
        defaultCardType = CardType.qr;
      }
    }

    return MemberInfo(
      flutterMemberInfo.memberQrCode,
      flutterMemberInfo.memberId ?? "",
      flutterMemberInfo.phone ?? "",
      flutterMemberInfo.memberCardId,
      flutterMemberInfo.name ?? "",
      flutterMemberInfo.isActive ?? false,
      flutterMemberInfo.point ?? 0,
      flutterMemberInfo.tierPoint,
      flutterMemberInfo.keepTierUntil,
      flutterMemberInfo.accumulationFrom,
      flutterMemberInfo.accumulationTo,
      flutterMemberInfo.currentTierCode,
      flutterMemberInfo.currentTierName,
      flutterMemberInfo.currentTierMinPoint,
      flutterMemberInfo.nextTierName,
      flutterMemberInfo.nextTierMinPoint,
      defaultCardType,
    );
  }

  static EncryptedMemCode toModelEncryptedMemCode(FlutterMembershipQr membershipQr) {
    return EncryptedMemCode(
      membershipQr.code ?? "",
      membershipQr.issueAt ?? 0,
      membershipQr.expireAt ?? 0,
    );
  }

  static NetworkConfigResult toModelNetworkConfigResult(FlutterNetworkData networkData) {
    final networkInfo = Network(
      networkData.networkInfo?.id ?? "",
      networkData.networkInfo?.name ?? "",
    );

    final flutterNetworkConfig = networkData.networkConfig;
    final flutterTiers = networkData.networkTiers;

    final networkConfig = NetworkConfig(
      flutterNetworkConfig?.exchangeRate ?? 0,
      flutterNetworkConfig?.currencyImg ?? "",
      flutterNetworkConfig?.currencyName ?? "",
      flutterNetworkConfig?.cardDefaultImg,
      flutterNetworkConfig?.avatarDefaultImg,
      flutterNetworkConfig?.allowToGetMemberByPhone,
      flutterNetworkConfig?.faqWebUrl,
      flutterNetworkConfig?.benefitWebUrl,
      flutterNetworkConfig?.brandColor,
      flutterNetworkConfig?.allowMerchantIssueMemberCard,
    );

    final networkTiers = flutterTiers
        ?.map((e) => Tier(
              e?.code ?? "",
              e?.name ?? "",
              e?.fromPoint ?? 1,
              e?.cardTieredImg,
              e?.tierIconImg,
            ))
        .toList();

    return NetworkConfigResult(networkInfo, networkConfig, networkTiers);
  }

  static TransactionHistory toModelTransactionHistory(FlutterTransactionHistory result) {
    final transactions = result.transactions?.map((e) {
      TransactionType txnType;
      switch (e?.txnType) {
        case FlutterTransactionType.grant:
          txnType = TransactionType.grant;
          break;
        case FlutterTransactionType.spend:
          txnType = TransactionType.spend;
          break;
        case FlutterTransactionType.returnGrant:
          txnType = TransactionType.returnGrant;
          break;
        case FlutterTransactionType.returnSpend:
          txnType = TransactionType.returnSpend;
          break;
        case FlutterTransactionType.initiate:
          txnType = TransactionType.initiate;
          break;
        case FlutterTransactionType.initiatePromotion:
          txnType = TransactionType.initiatePromotion;
          break;
        case FlutterTransactionType.issue:
          txnType = TransactionType.issue;
          break;
        default:
          txnType = TransactionType.grant;
          break;
      }

      AccountingType accountingType;
      switch (e?.accountingType) {
        case FlutterAccountingType.debit:
          accountingType = AccountingType.debit;
          break;
        default:
          accountingType = AccountingType.credit;
          break;
      }

      Status status;
      switch (e?.status) {
        case FlutterStatus.initFailure:
          status = Status.initFailure;
          break;
        case FlutterStatus.noAccouning:
          status = Status.noAccouning;
          break;
        case FlutterStatus.accountingSuccess:
          status = Status.accountingSuccess;
          break;
        case FlutterStatus.accountingFailure:
          status = Status.accountingFailure;
          break;
        case FlutterStatus.accouting:
          status = Status.accouting;
          break;
        default:
          status = Status.init;
          break;
      }

      return Transaction(
        e?.id ?? "",
        e?.refId,
        e?.orderCode,
        txnType,
        e?.txnTypeText ?? "",
        e?.txnDesc,
        accountingType,
        status,
        e?.point ?? 0,
        e?.createdAt ?? 0,
        e?.createdBy ?? "",
        e?.networkId ?? 0,
        e?.networkName ?? "",
        e?.merchantCode,
        e?.merchantName,
        e?.terminalCode,
        e?.terminalName,
        e?.terminalAddress,
      );
    });

    final pageData = PageData(
      result.pageData?.currentPage ?? 0,
      result.pageData?.totalPage ?? 0,
      result.pageData?.pageSize ?? 0,
      result.pageData?.totalItems ?? 0,
    );

    return TransactionHistory(transactions?.toList() ?? [], pageData);
  }

  static FlutterRegisterMemeberRequest toFlutterRegisterMemberRequest(
      RegisterMemeberRequest request) {
    FlutterGender gender;
    switch (request.gender) {
      case Gender.male:
        gender = FlutterGender.male;
        break;
      case Gender.female:
        gender = FlutterGender.female;
        break;
      default:
        gender = FlutterGender.none;
        break;
    }

    return FlutterRegisterMemeberRequest(
      phone: request.phone,
      name: request.name,
      address: request.address,
      dateOfBirth: request.dateOfBirth,
      email: request.email,
      gender: gender,
      idCardNumber: request.idCardNumber,
      idCitizenNumber: request.idCitizenNumber,
      passportNumber: request.passportNumber,
    );
  }

  static FlutterTransactionHistoryRequest toFlutterTransactionHistoryRequest(
      TransactionHistoryRequest request) {
    final txnTypes = request.txnTypes?.map((e) {
      String type;
      switch (e) {
        case TransactionType.grant:
          type = "01";
          break;
        case TransactionType.spend:
          type = "02";
          break;
        case TransactionType.returnGrant:
          type = "03";
          break;
        case TransactionType.returnSpend:
          type = "04";
          break;
        case TransactionType.initiate:
          type = "05";
          break;
        case TransactionType.initiatePromotion:
          type = "06";
          break;
        case TransactionType.issue:
          type = "07";
          break;
      }

      return type;
    });

    FlutterStatus? status;
    switch (request.status) {
      case Status.init:
        status = FlutterStatus.init;
        break;
      case Status.initFailure:
        status = FlutterStatus.initFailure;
        break;
      case Status.noAccouning:
        status = FlutterStatus.noAccouning;
        break;
      case Status.accountingSuccess:
        status = FlutterStatus.accountingSuccess;
        break;
      case Status.accountingFailure:
        status = FlutterStatus.accountingFailure;
        break;
      case Status.accouting:
        status = FlutterStatus.accouting;
        break;
      default:
        status = null;
    }

    return FlutterTransactionHistoryRequest(
      merchantCode: request.merchantCode,
      fromTime: request.fromTime,
      toTime: request.toTime,
      txnTypes: txnTypes?.toList(),
      status: status,
      page: request.page,
      pageSize: request.pageSize,
    );
  }
}
